@@ -14,7 +14,8 @@
 */
 #include "general.h"  /* must always come first */
 
-#include "parse.h"
+#include "types.h"
+#include "vstring.h"
 
 #define KEYWORD_NONE -1
 
@@ -23,6 +24,7 @@
 */
 extern void addKeyword (const char *const string, langType language, int value);
 extern int lookupKeyword (const char *const string, langType language);
+extern int lookupCaseKeyword (const char *const string, langType language);
 extern void freeKeywordTable (void);
 #ifdef DEBUG
 extern void printKeywordTable (void);
