@@ -1,8 +1,7 @@
 /*
  *      geany.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005 The Geany contributors
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -42,11 +41,11 @@ G_BEGIN_DECLS
 #define GEANY_FILEDEFS_SUBDIR			"filedefs"
 #define GEANY_TEMPLATES_SUBDIR			"templates"
 #define GEANY_TAGS_SUBDIR				"tags"
-#define GEANY_CODENAME					"Bemos"
-#define GEANY_HOMEPAGE					"http://www.geany.org/"
-#define GEANY_WIKI						"http://wiki.geany.org/"
-#define GEANY_BUG_REPORT				"http://www.geany.org/Support/Bugs"
-#define GEANY_DONATE					"http://www.geany.org/service/donate/"
+#define GEANY_CODENAME					"Wolbam"
+#define GEANY_HOMEPAGE					"https://www.geany.org/"
+#define GEANY_WIKI						"https://wiki.geany.org/"
+#define GEANY_BUG_REPORT				"https://www.geany.org/Support/Bugs"
+#define GEANY_DONATE					"https://www.geany.org/service/donate/"
 #define GEANY_STRING_UNTITLED			_("untitled")
 #define GEANY_DEFAULT_DIALOG_HEIGHT		350
 #define GEANY_WINDOW_DEFAULT_WIDTH		900
