@@ -1043,6 +1043,7 @@ void highlighting_init_styles(guint filetype_idx, GKeyFile *config, GKeyFile *co
 		init_styleset_case(RUBY);
 		init_styleset_case(RUST);
 		init_styleset_case(SH);
+		init_styleset_case(SMALLTALK);
 		init_styleset_case(SQL);
 		init_styleset_case(TCL);
 		init_styleset_case(TXT2TAGS);
@@ -1131,6 +1132,7 @@ void highlighting_set_styles(ScintillaObject *sci, GeanyFiletype *ft)
 		styleset_case(RUBY);
 		styleset_case(RUST);
 		styleset_case(SH);
+		styleset_case(SMALLTALK);
 		styleset_case(SQL);
 		styleset_case(TCL);
 		styleset_case(TXT2TAGS);
