@@ -1191,7 +1191,8 @@ static void recent_project_activate_cb(GtkMenuItem *menuitem, G_GNUC_UNUSED gpoi
 	gchar *utf8_filename = ui_menu_item_get_text(menuitem);
 	gchar *locale_filename = utils_get_locale_from_utf8(utf8_filename);
 
-	if (project_ask_close() && project_load_file_with_session(locale_filename))
+	if (app->project && !project_close(FALSE)) {}
+	else if (project_load_file_with_session(locale_filename))
 		recent_file_loaded(utf8_filename, recent_get_recent_projects());
 
 	g_free(locale_filename);
@@ -2490,6 +2491,11 @@ void ui_init_builder(void)
 			g_warning("Unable to get name from GtkBuilder object");
 			continue;
 		}
+		else
+		{
+			/* Set a default name for use from CSS by name */
+			gtk_widget_set_name(widget, name);
+		}
 
 		toplevel = ui_get_top_parent(widget);
 		if (toplevel)
