@@ -1308,12 +1308,10 @@ void utils_free_pointers(gsize arg_count, ...)
 }
 
 
-/* currently unused */
-#if 0
 /* Creates a string array deep copy of a series of non-NULL strings.
- * The first argument is nothing special.
- * The list must be ended with NULL.
- * If first is NULL, NULL is returned. */
+ * The first argument is nothing special and must not be NULL.
+ * The list must be terminated with NULL. */
+GEANY_EXPORT_SYMBOL
 gchar **utils_strv_new(const gchar *first, ...)
 {
 	gsize strvlen, i;
@@ -1343,7 +1341,6 @@ gchar **utils_strv_new(const gchar *first, ...)
 	strv[i] = NULL;
 	return strv;
 }
-#endif
 
 
 /**
@@ -2053,11 +2050,12 @@ gchar **utils_strv_join(gchar **first, gchar **second)
  * @return The common prefix that is part of all strings (maybe empty), or NULL if an empty list
  *         was passed in.
  */
-static gchar *utils_strv_find_common_prefix(gchar **strv, gssize strv_len)
+GEANY_EXPORT_SYMBOL
+gchar *utils_strv_find_common_prefix(gchar **strv, gssize strv_len)
 {
 	gsize num;
 
-	if (!NZV(strv))
+	if (strv_len == 0)
 		return NULL;
 
 	num = (strv_len == -1) ? g_strv_length(strv) : (gsize) strv_len;
@@ -2087,7 +2085,8 @@ static gchar *utils_strv_find_common_prefix(gchar **strv, gssize strv_len)
  *
  * @return The common prefix that is part of all strings.
  */
-static gchar *utils_strv_find_lcs(gchar **strv, gssize strv_len)
+GEANY_EXPORT_SYMBOL
+gchar *utils_strv_find_lcs(gchar **strv, gssize strv_len, const gchar *delim)
 {
 	gchar *first, *_sub, *sub;
 	gsize num;
@@ -2097,10 +2096,11 @@ static gchar *utils_strv_find_lcs(gchar **strv, gssize strv_len)
 	char *lcs;
 	gsize found;
 
-	num = (strv_len == -1) ? g_strv_length(strv) : (gsize) strv_len;
-	if (num < 1)
+	if (strv_len == 0)
 		return NULL;
 
+	num = (strv_len == -1) ? g_strv_length(strv) : (gsize) strv_len;
+
 	first = strv[0];
 	len = strlen(first);
 
@@ -2113,8 +2113,18 @@ static gchar *utils_strv_find_lcs(gchar **strv, gssize strv_len)
 		/* No point in continuing if the remainder is too short */
 		if (max > chars_left)
 			break;
+		/* If delimiters are given, we only need to compare substrings which start and
+		 * end with one of them, so skip any non-delim chars at front ... */
+		if (NZV(delim) && (strchr(delim, _sub[0]) == NULL))
+			continue;
 		for (n_chars = 1; n_chars <= chars_left; n_chars++)
 		{
+			if (NZV(delim))
+			{	/* ... and advance to the next delim char at the end, if any */
+				if (!_sub[n_chars] || strchr(delim, _sub[n_chars]) == NULL)
+					continue;
+				n_chars += 1;
+			}
 			g_strlcpy(sub, _sub, n_chars+1);
 			found = 1;
 			for (gsize i = 1; i < num; i++)
@@ -2159,86 +2169,71 @@ gchar **utils_strv_shorten_file_list(gchar **file_names, gssize file_names_len)
 {
 	gsize num;
 	gsize i;
-	gchar *prefix, *substring, *lcs;
-	gchar *start, *end;
+	gchar *prefix, *lcs, *end;
 	gchar **names;
-	gsize prefix_len, lcs_len;
+	gsize prefix_len = 0, lcs_len = 0;
 
-	/* We don't dereference file_names if file_names_len == 0. */
-	g_return_val_if_fail(file_names_len == 0 || file_names != NULL, NULL);
+	if (file_names_len == 0)
+		return g_new0(gchar *, 1);
+
+	g_return_val_if_fail(file_names != NULL, NULL);
 
-	/* The return value shall have exactly the same size as the input. If the input is a
-	 * GStrv (last element is NULL), the output will follow suit. */
 	num = (file_names_len == -1) ? g_strv_length(file_names) : (gsize) file_names_len;
+	/* Always include a terminating NULL, enables easy freeing with g_strfreev()
+	 * We just copy the pointers so we can advance them here. But don't
+	 * forget to duplicate the strings before returning.
+	 */
+	names = g_new(gchar *, num + 1);
+	memcpy(names, file_names, num * sizeof(gchar *));
 	/* Always include a terminating NULL, enables easy freeing with g_strfreev() */
-	names = g_new0(gchar *, num + 1);
+	names[num] = NULL;
 
-	prefix = utils_strv_find_common_prefix(file_names, num);
 	/* First: determine the common prefix, that will be stripped.
-	 * Don't strip single-letter prefixes, such as '/' */
-	prefix_len = 0;
-	if (NZV(prefix) && prefix[1])
+	 * We only want to strip full path components, including the trailing slash.
+	 * Except if the component is just "/".
+	 */
+	prefix = utils_strv_find_common_prefix(names, num);
+	end = strrchr(prefix, G_DIR_SEPARATOR);
+	if (end && end > prefix)
 	{
-		/* Only strip directory components, include trailing '/' */
-		start = strrchr(prefix, G_DIR_SEPARATOR);
-		if (start)
-			prefix_len = start - prefix + 1;
+		prefix_len = end - prefix + 1; /* prefix_len includes the trailing slash */
+		for (i = 0; i < num; i++)
+			names[i] += prefix_len;
 	}
 
-	/* Second: determine the longest common substring (lcs), that will be ellipsized. Look
-	 * only at the directory parts since we don't want the file name to be detected as the lcs.
-	 * Also, the first component cannot be common (since it would be part of the common prefix),
-	 * so ignore that as well.
-	 * Surround with dir separators so that we can easily determine the boundaries for ellipsizing. */
-	for (i = 0; i < num; i++) {
-		start = strchr(file_names[i] + prefix_len, G_DIR_SEPARATOR);
-		end = start ? strrchr(start+1, G_DIR_SEPARATOR) : NULL;
-		/* Breaking out early will also skip looking for lcs (wouldn't succeed anyway). */
-		if (!start || !end)
-			break;
-		names[i] = g_strndup(start, end-start+1);
-	}
-
-	lcs_len = 0;
-	substring = (i == num) ? utils_strv_find_lcs(names, num) : NULL;
-	if (NZV(substring))
+	/* Second: determine the longest common substring (lcs), that will be ellipsized. Again,
+	 * we look only for full path compnents so that we ellipsize between separators. This implies
+	 * that the file name cannot be ellipsized which is desirable anyway.
+	 */
+	lcs = utils_strv_find_lcs(names, num, G_DIR_SEPARATOR_S"/");
+	if (lcs)
 	{
-		/* Strip leading component. */
-		start = strchr(substring, G_DIR_SEPARATOR);
-		if (start)
-		{
-			lcs = start + 1;
-			/* Strip trailing components (perhaps empty). */
-			end = strrchr(lcs, G_DIR_SEPARATOR);
-			if (end)
-			{
-				*end = '\0';
-				lcs_len = strlen(lcs);
-				/* Don't bother for tiny common parts (which are often just "." or "/"). */
-				if (lcs_len < 5)
-					lcs_len = 0;
-			}
-		}
+		lcs_len = strlen(lcs);
+		/* Don't bother for tiny common parts (which are often just "." or "/"). Beware
+		 * that lcs includes the enclosing dir separators so the part must be at least 5 chars
+		 * to be eligible for ellipsizing.
+		 */
+		if (lcs_len < 7)
+			lcs_len = 0;
 	}
 
-	/* Finally build the shortened list of unique file names */
+	/* Last: build the shortened list of unique file names */
 	for (i = 0; i < num; i++)
 	{
-		start = file_names[i] + prefix_len;
 		if (lcs_len == 0)
 		{	/* no lcs, copy without prefix */
-			SETPTR(names[i], g_strdup(start));
+			names[i] = g_strdup(names[i]);
 		}
 		else
 		{
-			const gchar *lcs_start = strstr(start, lcs);
+			const gchar *lcs_start = strstr(names[i], lcs);
 			const gchar *lcs_end = lcs_start + lcs_len;
-			/* Maybe replace with unicode's "…" ? */
-			SETPTR(names[i], g_strdup_printf("%.*s...%s", (int)(lcs_start - start), start, lcs_end));
+			/* Dir seperators are included in lcs but shouldn't be elipsized. */
+			names[i] = g_strdup_printf("%.*s...%s", (int)(lcs_start - names[i] + 1), names[i], lcs_end - 1);
 		}
 	}
 
-	g_free(substring);
+	g_free(lcs);
 	g_free(prefix);
 
 	return names;
@@ -2377,13 +2372,13 @@ void utils_start_new_geany_instance(const gchar *doc_path)
 
 		if (!utils_spawn_async(NULL, (gchar**) argv, NULL, 0, NULL, NULL, NULL, &err))
 		{
-			g_printerr("Unable to open new window: %s", err->message);
+			g_printerr("Unable to open new window: %s\n", err->message);
 			g_error_free(err);
 		}
 		g_free(exec_path);
 	}
 	else
-		g_printerr("Unable to find 'geany'");
+		g_printerr("Unable to find 'geany'\n");
 }
 
 
@@ -2411,3 +2406,71 @@ gchar *utils_get_real_path(const gchar *file_name)
 {
 	return tm_get_real_path(file_name);
 }
+
+
+/*
+ * Get a string describing the OS.
+ *
+ * If the OS can be determined, a string which describes the OS will
+ * be returned. If no OS can be determined then `NULL` will be returned.
+ *
+ * @note The format of the returned string is unspecified and is only
+ * meant to provide diagnostic information to the user.
+ *
+ * @return A newly-allocated string containing a description of the
+ * OS if it can be determined or `NULL` if it cannot.
+ *
+ * @since 1.37
+ */
+gchar *utils_get_os_info_string(void)
+{
+	gchar *os_info = NULL;
+
+#if GLIB_CHECK_VERSION(2, 64, 0)
+# if ! defined(__APPLE__)
+	/* on non-macOS operating systems */
+	{
+		GString *os_str;
+		gchar *pretty_name;
+		gchar *code_name;
+
+		pretty_name = g_get_os_info(G_OS_INFO_KEY_PRETTY_NAME);
+		if (pretty_name == NULL)
+			return NULL;
+
+		os_str = g_string_new(pretty_name);
+		g_free(pretty_name);
+
+		code_name = g_get_os_info(G_OS_INFO_KEY_VERSION_CODENAME);
+		if (code_name != NULL)
+		{
+			g_string_append_printf(os_str, " (%s)", code_name);
+			g_free(code_name);
+		}
+
+		os_info = g_string_free(os_str, FALSE);
+	}
+# else
+	/* on macOS, only `G_OS_INFO_KEY_NAME` is supported and returns the
+	 * fixed string `macOS` */
+	os_info = g_get_os_info(G_OS_INFO_KEY_NAME);
+# endif
+#else
+	/* if g_get_os_info() is not available, do it the old-fashioned way */
+# if defined(_WIN64)
+	os_info = g_strdup("Microsoft Windows (64-bit)");
+# elif defined(_WIN32)
+	os_info = g_strdup("Microsoft Windows");
+# elif defined(__APPLE__)
+	os_info = g_strdup("Apple macOS");
+# elif defined(__linux__)
+	os_info = g_strdup("Linux");
+# elif defined(__FreeBSD__)
+	os_info = g_strdup("FreeBSD");
+# elif defined(__ANDROID__)
+	os_info = g_strdup("Android");
+# endif
+#endif
+
+	return os_info;
+}
